/*
Module : DTime+BaseCalendar.H
Purpose: Defines the interface for the DTime+ CBaseCalendar class
Created: PJN / 01-05-2006
History: PJN / 20-05-2006 1. Canonical DaysInMonth method now takes a Year and a Month parameter instead of a
                          month and a leap year boolean. This is required for support of the Hebrew calendar
                          where knowing whether a year is a leap year or not is not good enough to determine
                          the number of days in any month. This is because of the more complex nature of the 
                          hebrew calendar and its' postponement rules for the Hebrew New Year (Rosh ha-Shannah).
                          Please note that this means you should examine all calls to this method in client
                          code to ensure it is updated correctly.
                          2. Canonical DaysInYear method layout has been updated to take a year instead of a
                          leap year boolean. The reason is as for item one above. Again you should examine all
                          calls to this method in client code to ensure it is updated correctly.
                          3. Canonical DaysSinceEndPreviousYear and DaysSinceFirstOfYear parameter layout 
                          has been updated. Both now take a year, month and day indicator. The reason is as 
                          for item one above. Again you should examine all calls to this method in client code 
                          to ensure it is updated correctly.

Copyright (c) 2006 - 2007 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////// Macros / Defines //////////////////////////////////

#ifndef __DTIMEPLUSBASECALENDAR_H__
#define __DTIMEPLUSBASECALENDAR_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/////////////////////////// Includes //////////////////////////////////////////

#include "DTime+Defines.h"


/////////////////////////// Classes ///////////////////////////////////////////

namespace DTimePlus
{

//This is the canonical class definition which defines the calendar traits class used by CDate and CLDate
class CBaseCalendar
{
public:
//Methods
  static bool  DMYToLong(long Year, Month month, WORD Day, long& lDays, bool bDoAssert);
  static WORD  DaysInMonth(long Year, Month month);
  static WORD  DaysInYear(long Year);
  static DateS GetDate(long Days);
  static bool  IsLeap(long Year);
  static WORD  DaysSinceEndPreviousYear(long Year, WORD month, WORD Day);
  static WORD  DaysSinceFirstOfYear(long Year, WORD month, WORD Day);
};

} //namespace DTimePlus

#endif //__DTIMEPLUSBASECALENDAR_H__
