/*
Module : DTime+Helpers.H
Purpose: Defines the interface for the various helper functions for DTime+
Created: PJN / 01-05-2006
History: PJN / 25-05-2006 1. Optimized the DTimePlus::WeekOfDayModulo and DTimePlus::lfloor functions. This
                          results in a 2 fold increase in the speed of the key algorithms for the Julian
                          and Gregorian calendars.
         PJN / 27-05-2006 1. Removed the need for the function WeekOfDaysModulo
         PJN / 02-07-2006 1. Updated the code to clean compile on VC 2005     
                          2. Code now uses new C++ style casts rather than old style C casts where necessary.

Copyright (c) 2006 - 2007 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////// Macros / Defines //////////////////////////////////

#ifndef __DTIMEPLUSHELPERS_H__
#define __DTIMEPLUSHELPERS_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/////////////////////////// Includes //////////////////////////////////////////

#include "DTime+Defines.h"
#include <malloc.h> //for _alloca support
#include <math.h> 


/////////////////////////// Classes ///////////////////////////////////////////

namespace DTimePlus 
{
  static CString SafeGetLocaleInfo(LCID Locale, LCTYPE LCType)
  {
    //What will be the return value from this function
    CString sInfo;

    //First ask the API to give us the buffer size
    int nChars = GetLocaleInfo(Locale, LCType, NULL, 0);
    BOOL bWantStop = FALSE;
    while (nChars && !bWantStop)
    {
      //alloc the buffer to the required size
      LPTSTR pszBuffer = static_cast<LPTSTR>(_alloca(nChars * sizeof(TCHAR)));
      nChars = GetLocaleInfo(Locale, LCType, pszBuffer, nChars);
      if (nChars == 0)
      {
        if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
        {
          //Expand the buffer, and loop around again
          nChars *= 2;
        }
        else
          bWantStop = TRUE;
      }
      else
      {
        sInfo = pszBuffer;
        bWantStop = TRUE;
      }
    }

    return sInfo;
  }

  static CString SafeGetDateFormat(LCID Locale, DWORD dwFlags, SYSTEMTIME* pSystemTime)
  {
    //What will be the return value from this function
    CString sInfo;

    //First ask the API to give us the buffer size
    int nChars = ::GetDateFormat(Locale, dwFlags, pSystemTime, NULL, NULL, 0);
    BOOL bWantStop = FALSE;
    while (nChars && !bWantStop)
    {
      //alloc the buffer to the required size
      LPTSTR pszBuffer = static_cast<LPTSTR>(_alloca(nChars * sizeof(TCHAR)));
      nChars = ::GetDateFormat(Locale, dwFlags, pSystemTime, NULL, pszBuffer, nChars);
      if (nChars == 0)
      {
        if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
        {
          //Expand the buffer, and loop around again
          nChars *= 2;
        }
        else
          bWantStop = TRUE;
      }
      else
      {
        sInfo = pszBuffer;
        bWantStop = TRUE;
      }
    }

    return sInfo;
  }

  static CString SafeGetTimeFormat(LCID Locale, DWORD dwFlags, SYSTEMTIME* pSystemTime)
  {
    //What will be the return value from this function
    CString sInfo;

    //First ask the API to give us the buffer size
    int nChars = ::GetTimeFormat(Locale, dwFlags, pSystemTime, NULL, NULL, 0);
    BOOL bWantStop = FALSE;
    while (nChars && !bWantStop)
    {
      //alloc the buffer to the required size
      LPTSTR pszBuffer = static_cast<LPTSTR>(_alloca(nChars * sizeof(TCHAR)));
      nChars = ::GetTimeFormat(Locale, dwFlags, pSystemTime, NULL, pszBuffer, nChars);
      if (nChars == 0)
      {
        if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
        {
          //Expand the buffer, and loop around again
          nChars *= 2;
        }
        else
          bWantStop = TRUE;
      }
      else
      {
        sInfo = pszBuffer;
        bWantStop = TRUE;
      }
    }

    return sInfo;
  }

  inline static double dmod(double x, double y) 
  {
		return x - y * floor(x / y);
	}

  inline static long lmod(long x, long y) 
  {
    if (x >= 0)
    {
      if (y >= 0)
        return x%y;
      else
        return y - ((-x)%-y);
    }
    else 
    {
      if (y >= 0)
        return (y - ((-x)%y)) % y;
      else
        return -(-x%-y);
    }
	}

	inline static long dquotient(double x, double y) 
  {
		return static_cast<long>(floor(x / y));
	}

  inline static long lquotient(long x, long y)
  {
    ASSERT(y > 0);
    return (x >= 0 ? x/y : (x%y == 0) - 1 - (-x)/y);
  }

} //namespace DTimePlus

#endif //__DTIMEPLUSHELPERS_H__
