/*
Module : DTime+JulianGregorianHolidays.H
Purpose: Defines the interface for the DTime+ CJulianGregorianHolidays class which defines various methods
         to create holidays, feasts, holy days, Bank Holidays etc in the Julian or Gregorian calendar
Created: PJN / 29-05-2006
History: None

Copyright (c) 2006 - 2007 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////// Macros / Defines //////////////////////////////////

#ifndef __DTIMEPLUSJULIANGREGORIANHOLIDAYS_H__
#define __DTIMEPLUSJULIANGREGORIANHOLIDAYS_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/////////////////////////// Includes //////////////////////////////////////////

#include "DTime+Defines.h"


/////////////////////////// Classes ///////////////////////////////////////////

namespace DTimePlus
{

template <class TCALENDAR = CGregorianCalendar>
class CJulianGregorianHolidays
{
public:
//Well known worldwide holidays
  inline static CDate<TCALENDAR> NewYearsDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, January, 1);
  }

  inline static CDate<TCALENDAR> AprilFoolsDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, April, 1);
  }

  inline static CDate<TCALENDAR> Halloween(long Year) 
  {
    return CDate<TCALENDAR>(Year, October, 31);
  }


//Saints Feast Days
  inline static CDate<TCALENDAR> StValentinesDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, February, 14);
  }

  inline static CDate<TCALENDAR> StPatricksDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, March, 17);
  }

  inline static CDate<TCALENDAR> StAndrewsDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, November, 30);
  }

  inline static CDate<TCALENDAR> StPaulsDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, January, 25);
  }

  inline static CDate<TCALENDAR> StGeorgesDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, April, 23);
  }

  inline static CDate<TCALENDAR> StDavidsDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, March, 1);
  }

  inline static CDate<TCALENDAR> StStephensDays(long Year)
  {
    return CDate<TCALENDAR>(Year, December, 26);
  }


//Other National holidays
  inline static CDate<TCALENDAR> CanadaDay(long Year)
  {
    return CDate<TCALENDAR>(Year, July, 1);
  }

  inline static CDate<TCALENDAR> BastilleDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, July, 14);
  }

  inline static CDate<TCALENDAR> UKBoxingDay(long Year)
  {
    return CDate<TCALENDAR>(Year, December, 26); //Aka St. Stephens day in Ireland
  }


//US Specific Holidays
  inline static CDate<TCALENDAR> MartinLutherKingDay(long Year)
  {
    return CDate<TCALENDAR>(Year, January, 3, Monday);
  }  

  inline static CDate<TCALENDAR> AbrahamLincolnsBirthday(long Year)
  {
    return CDate<TCALENDAR>(Year, February, 12);
  }  

  inline static CDate<TCALENDAR> GeorgeWashingtonsBirthday(long Year)
  {
    return CDate<TCALENDAR>(Year, February, 22);
  }  

  inline static CDate<TCALENDAR> USPresidentsDay(long Year)
  {
    if (Year >= 1971)
      return CDate<TCALENDAR>(Year, February, 3, Monday);
    else
      return CDate<TCALENDAR>(Year, February, 22);
  }  

  inline static CDate<TCALENDAR> USArmedForcesDay(long Year)
  {
    return CDate<TCALENDAR>(Year, May, 3, Sunday);
  }  

  inline static CDate<TCALENDAR> USMemorialDay(long Year)
  {
    if (Year >= 1971)
      return CDate<TCALENDAR>(Year, May, 5, Monday);
    else
      return CDate<TCALENDAR>(Year, May, 30);
  }  

  inline static CDate<TCALENDAR> USFlagDay(long Year)
  {
    return CDate<TCALENDAR>(Year, June, 14);
  }  

  inline static CDate<TCALENDAR> USIndependenceDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, July, 4);
  }

  inline static CDate<TCALENDAR> USLaborDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, September, 1, Monday);
  }

  inline static CDate<TCALENDAR> USColumbusDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, October, 2, Monday);
  }

  inline static CDate<TCALENDAR> USElectionDay(long Year) 
  {
    CDate<TCALENDAR> ElectionDay(Year, November, 2);
    ElectionDay += ((Tuesday + 7 - ElectionDay.GetDayOfWeek()) % 7);

    return ElectionDay;
  }

  inline static CDate<TCALENDAR> USVeteransDay(long Year) 
  {
    if (Year >= 1971 && Year <= 1977)
      return CDate<TCALENDAR>(Year, October, 4, Monday);
    else
      return CDate<TCALENDAR>(Year, November, 11);
  }

  inline static CDate<TCALENDAR> USThanksgivingDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, November, 4, Thursday);
  }

  inline static CDate<TCALENDAR> GroundhogDay(long Year) 
  {
    return CDate<TCALENDAR>(Year, February, 2);
  }


//Christian Feast Days
  inline static CDate<TCALENDAR> Epiphany(long Year)  //Note DTime+ uses the traditional Roman catholic date for Epiphany, instead of the 
  {                                                   //newer definition which is the First Sunday after January 1st. For more info
    return CDate<TCALENDAR>(Year, January, 6);        //please see http://en.wikipedia.org/wiki/Epiphany
  }

  inline static CDate<TCALENDAR> Candlemas(long Year)  
  {
    return CDate<TCALENDAR>(Year, February, 2);
  }

  inline static CDate<TCALENDAR> ShroveTuesday(long Year)
  {                  
    return EasterSunday(Year) - 47; //Shrove Tuesday occurs 47 days before Easter Sunday
  }

  inline static CDate<TCALENDAR> AshWednesday(long Year)
  {                  
    return EasterSunday(Year) - 46; //Ash Wednesday occurs 46 days before Easter Sunday
  }

  inline static CDate<TCALENDAR> FeastOfTheAnnunciation(long Year)
  { 
    return CDate<TCALENDAR>(Year, March, 25);
  }

  inline static CDate<TCALENDAR> PalmSunday(long Year)
  { 
    return EasterSunday(Year) - 7; //Palm Sunday occurs 7 days before Easter Sunday
  }

  inline static CDate<TCALENDAR> MaundyThursday(long Year)
  { 
    return EasterSunday(Year) - 3; //Maundy (or Holy) Thursday occurs 3 days before Easter Sunday
  }

  inline static CDate<TCALENDAR> GoodFriday(long Year)
  { 
    return EasterSunday(Year) - 2; //Good Friday occurs 2 days before Easter Sunday
  }

  inline static CDate<TCALENDAR> EasterSunday(long Year)
  {
    //Call the helper function to calculate Easter Sunday for us
    WORD wMonth;
    WORD wDay;
    TCALENDAR::EasterSunday(Year, wMonth, wDay);

    return CDate<TCALENDAR>(Year, wMonth, wDay);
  }

  inline static CDate<TCALENDAR> FeastOfTheAscension(long Year)
  { 
    return EasterSunday(Year) + 39; //Ascension Day occurs 39 days after Easter Sunday
  }

  inline static CDate<TCALENDAR> Pentecost(long Year)
  { 
    return EasterSunday(Year) + 49; //Pentecost occurs 49 days after Easter Sunday
  }

  inline static CDate<TCALENDAR> FeastOfTheAssumption(long Year)  
  {
    return CDate<TCALENDAR>(Year, August, 15);
  }

  inline static CDate<TCALENDAR> AllSaintsDay(long Year)  
  {
    return CDate<TCALENDAR>(Year, November, 1);
  }

  inline static CDate<TCALENDAR> AllSoulsDay(long Year)  
  {
    return CDate<TCALENDAR>(Year, November, 2);
  }

  inline static CDate<TCALENDAR> FeastOfTheImmaculateConception(long Year)  
  {
    return CDate<TCALENDAR>(Year, December, 8);
  }

  inline static CDate<TCALENDAR> ChristmasDay(long Year)  
  {
    return CDate<TCALENDAR>(Year, December, 25);
  }
};

} //namespace DTimePlus

#endif //__DTIMEPLUSJULIANGREGORIANHOLIDAYS_H__
