/*
Module : DTIME+UI.H
Purpose: Defines the interface for UI support for the DTime+ class library
Created: PJN / 21-04-2006
History: PJN / 02-07-2006 1. Updated the code to clean compile on VC 2005     
                          2. Code now uses new C++ style casts rather than old style C casts where necessary.

Copyright (c) 2006 - 2007 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////// Macros / Defines //////////////////////////////////

#ifndef __DTIMEPLUSUI_H__
#define __DTIMEPLUSUI_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/////////////////////////// Includes //////////////////////////////////////////

#include "DTime+.h"  
                      

/////////////////////////// Functions / Classes ///////////////////////////////

void DDX_DateTimeCtrl(CDataExchange* pDX, int nIDC, DTimePlus::CDate<DTimePlus::CGregorianCalendar>& value)
{
	HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
	CWnd* pWnd = CWnd::FromHandle(hWndCtrl);
  ASSERT(pWnd);

	if (pDX->m_bSaveAndValidate)
  {
	  SYSTEMTIME sysTime;
	  LRESULT result = pWnd->SendMessage(DTM_GETSYSTEMTIME, 0, reinterpret_cast<LPARAM>(&sysTime));
	  if (result == GDT_VALID)
		  value.Set(sysTime);
	  else                 
      value.Set();
  }
	else
  {
	  SYSTEMTIME sysTime;
	  WPARAM wParam = GDT_NONE;
	  if (value.GetAsSystemTime(sysTime))
		  wParam = GDT_VALID;

	  pWnd->SendMessage(DTM_SETSYSTEMTIME, wParam, reinterpret_cast<LPARAM>(&sysTime));
  }
}

void DDX_DateTimeCtrl(CDataExchange* pDX, int nIDC, DTimePlus::CLTimeOfDay<DTimePlus::CLTimeSpan>& value)
{
	HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
	CWnd* pWnd = CWnd::FromHandle(hWndCtrl);
  ASSERT(pWnd);

	if (pDX->m_bSaveAndValidate)
  {
	  SYSTEMTIME sysTime;
	  LRESULT result = pWnd->SendMessage(DTM_GETSYSTEMTIME, 0, reinterpret_cast<LPARAM>(&sysTime));
	  if (result == GDT_VALID)
		  value.Set(sysTime);
	  else
      value.Set();
  }
	else
  {
	  SYSTEMTIME sysTime;
	  WPARAM wParam = GDT_NONE;
	  if (value.GetAsSystemTime(sysTime))
		  wParam = GDT_VALID;

	  pWnd->SendMessage(DTM_SETSYSTEMTIME, wParam, reinterpret_cast<LPARAM>(&sysTime));
  }
}

#endif //__DTIMEPLUSUI_H__
