#include "stdafx.h"
#include "DTimeTest.h"
#include "DTime+.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CWinApp theApp;

using namespace DTimePlus;

int _tmain(int /*argc*/, TCHAR* /*argv*/[], TCHAR* /*envp*/[])
{
	//initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		_tprintf(_T("Fatal Error: MFC initialization failed\n"));
		return 1;
	}

  DWORD StartTicks = GetTickCount();

  //Set the title of the console Window
  SetConsoleTitle(_T("Test program to exercise the DTime+ Class framework"));

  //Display some intro text
  _tprintf(_T("This program is best run under the Visual C debugger so that\n"));
  _tprintf(_T("the code can be single stepped and stepped into etc.\n"));
  _tprintf(_T("Some calls to DTime+ contained in this code is designed to cause ASSERTIONS\n"));
  _tprintf(_T("<Hit Enter to continue>\n"));
  int chr = getchar();

  ASSERT(DTimePlus::lmod(9, 5) == 4);
  ASSERT(DTimePlus::lmod(-9, 5) == 1);
  ASSERT(DTimePlus::lmod(9, -5) == -1);
  ASSERT(DTimePlus::lmod(-9, -5) == -4); 

  //Test of the CDate class
  CDate<CGregorianCalendar> xstr(1996, January, 1);
  CString sString = xstr.Format();


  //Test of the calendar constructor parameters
  CDate<CGregorianCalendar> x(1582, October, 15);
  CDate<CJulianCalendar> p = x.AsJulianCDate();
  sString = p.Format();
  CDate<CGregorianCalendar> p2 = x.AsGregorianCDate();
  sString = p2.Format();
  CDate<CJulianCalendar> x2 = CDate<CJulianCalendar>(1582, October, 15);
  p = x2.AsJulianCDate();
  sString = p.Format();
  p2 = x.AsGregorianCDate();
  sString = p2.Format();
  x = CDate<CGregorianCalendar>(1582, October, 15);
  p = x.AsJulianCDate();
  sString = p.Format();
  p2 = x.AsGregorianCDate();
  sString = p2.Format();
  x2 = CDate<CJulianCalendar>(1582, October, 4);
  p = x2.AsJulianCDate();
  sString = p.Format();
  p2 = x.AsGregorianCDate();
  sString = p2.Format();
  x2 = CDate<CJulianCalendar>(1582, October, 4);
  p = x2.AsJulianCDate();
  sString = p.Format();
  p2 = x.AsGregorianCDate();
  sString = p2.Format();
  x = CDate<CGregorianCalendar>(1582, October, 4);
  p = x.AsJulianCDate();
  sString = p.Format();
  p2 = x.AsGregorianCDate();
  sString = p2.Format();


  //first all the constructors
  x = CDate<CGregorianCalendar>(1582, October, 15);  //GDN = 0
  x = CDate<CGregorianCalendar>(1992, October, 10);  //GDN = 149745
  x2 = CDate<CJulianCalendar>(-4712, January, 1);  //GDN = -2299161
  x2 = CDate<CJulianCalendar>(1582, October, 4);   //GDN = -1;

  x = CDate<CGregorianCalendar>(2006, April, 22);  //GDN = 154687;
  DayOfWeek dow(x.GetDayOfWeek());
  sString = x.Format();

  //should be invalid dates, causing ASSERTs
  /*
  x = CDate(1582, October, 8, true);
  ASSERT(!x.IsValid());
  x = CDate(1995, February, 29, true);
  ASSERT(!x.IsValid());
  x = CDate(1900, February, 29, true);
  ASSERT(!x.IsValid());
  x = CDate(1582, January, 32, true);
  ASSERT(!x.IsValid());
  x = CDate(1900, static_cast<Month>(December + 1), 8, true);
  ASSERT(!x.IsValid());
  */


  //should be invalid dates and should ASSERT
  /*
#ifdef _DEBUG
  x = CDate(1582, CDate::October, 8, true);
  x = CDate(1995, CDate::February, 29, true);
  x = CDate(1900, CDate::February, 29, true);
  x = CDate(1582, January, 32, true);
  x = CDate(1900, static_cast<Month>(December + 1), 8, true);
#endif
  */


  //Construction using the Win32 SYSTEMTIME structure
  SYSTEMTIME st;
  st.wYear = 1582;
  st.wMonth = 10;
  st.wDay = 15;
  x = CDate<CGregorianCalendar>(st);
  CString s = x.Format();

  //Construction using the Win32 FILETIME structure
  GetLocalTime(&st);
  FILETIME ft;
  SystemTimeToFileTime(&st, &ft);
  x = CDate<CGregorianCalendar>(ft);
  s = x.Format();

  //Construction using a CTime
  CTime t(CTime::GetCurrentTime());
  x = CDate<CGregorianCalendar>(t);
  s = x.Format();


  //Construction using the direct count of days
  x = CDate<CGregorianCalendar>(0, EPOCH_JD);    //GDN = -2299161
  x = CDate<CGregorianCalendar>(0, EPOCH_GREG);  //GDN = 0
  x = CDate<CGregorianCalendar>(0, EPOCH_MJD);   //GDN = 100839
  x = CDate<CGregorianCalendar>(0, EPOCH_1900);  //GDN = 115860
  x = CDate<CGregorianCalendar>(0, EPOCH_1950);  //GDN = 134122
  x = CDate<CGregorianCalendar>(0, EPOCH_CTIME); //GDN = 141427
  x = CDate<CGregorianCalendar>(0, EPOCH_2000);  //GDN = 152384

  //CTime Construction
  CTime z(1970, 1, 2, 0, 0, 0);
  x = CDate<CGregorianCalendar>(z);  //GDN = 141428
  s = x.Format();
      
  //COleDateTime Construction
  COleDateTime dt(1992, October, 10, 0, 0, 0);
  x = CDate<CGregorianCalendar>(dt);  //GDN = 149745
  s = x.Format();
  dt = COleDateTime(1970, 1, 2, 0, 0, 0);
  x = CDate<CGregorianCalendar>(dt);  //GDN = 141428
  s = x.Format();


  //String constructors
  x = CDate<CGregorianCalendar>(_T("20/03/96"), _T("dmy2/"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("20/03/46"), _T("dmy2/"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("2/03/96"), _T("mdy2/"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("2-03-96"), _T("mdy2-"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("20/03/1496"), _T("dmy4/"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("20/03/2146"), _T("dmy4/"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("2/03/1896"), _T("mdy4/"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("2-03-1496"), _T("mdy4-"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("1496/20/03"), _T("ydm4/"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("2146/20/03"), _T("ydm4/"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("1896/2/03"), _T("ymd4/"));
  s = x.Format();
  x = CDate<CGregorianCalendar>(_T("1496-2-03"), _T("ymd4-"));
  s = x.Format();



  //Static Constructors
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  s = x.Format();
  _tprintf(_T("The current local date is %s\n"), s.operator LPCTSTR());
  s = x.AsJulianCDate().Format();
  _tprintf(_T("The current local date (in the Julian Calendar) is %s\n"), s.operator LPCTSTR());
  p = CDate<CJulianCalendar>::CurrentLocalDate();
  s = p.Format();
  _tprintf(_T("The current local date (in the Julian Calendar *) is %s\n"), s.operator LPCTSTR());

  x = CDate<CGregorianCalendar>::CurrentSystemDate();
  s = x.Format();
  _tprintf(_T("The current system date is %s\n"), s.operator LPCTSTR());
  s = x.AsJulianCDate().Format();
  _tprintf(_T("The current system date (in the Julian Calendar) is %s\n"), s.operator LPCTSTR());
  p = CDate<CJulianCalendar>::CurrentSystemDate();
  s = p.Format();
  _tprintf(_T("The current system date (in the Julian Calendar *) is %s\n"), s.operator LPCTSTR());

  x2 = CDate<CJulianCalendar>::JDEpoch();
  s = x2.Format();
  _tprintf(_T("The Julian Day Epoch is %s\n"), s.operator LPCTSTR());
  x = CDate<CGregorianCalendar>::MJDEpoch();
  s = x.Format();
  _tprintf(_T("The Modified Julian Day Epoch is %s\n"), s.operator LPCTSTR());
  x = CDate<CGregorianCalendar>::Epoch1900();
  s = x.Format();
  _tprintf(_T("The 1900 Epoch is %s\n"), s.operator LPCTSTR());
  x = CDate<CGregorianCalendar>::Epoch1950();
  s = x.Format();
  _tprintf(_T("The 1950 Epoch is %s\n"), s.operator LPCTSTR());
  x = CDate<CGregorianCalendar>::EpochCTime();
  s = x.Format();
  _tprintf(_T("The CTime Epoch is %s\n"), s.operator LPCTSTR());
  x = CDate<CGregorianCalendar>::Epoch2000();
  s = x.Format();
  _tprintf(_T("The 2000 Epoch is %s\n"), s.operator LPCTSTR());
  x = CDate<CGregorianCalendar>::GregorianEpoch();
  s = x.Format();
  _tprintf(_T("The Gregorian Epoch is %s\n"), s.operator LPCTSTR());


  //Try the static operations

  ASSERT(CGregorianCalendar::IsLeap(2000));
  ASSERT(!CGregorianCalendar::IsLeap(1900));
  ASSERT(!CGregorianCalendar::IsLeap(1902));
  ASSERT(CJulianCalendar::IsLeap(1400));
  ASSERT(CJulianCalendar::IsLeap(1300));
  ASSERT(!CJulianCalendar::IsLeap(1302));
  ASSERT(CGregorianCalendar::DaysInYear(2000) == 366);
  ASSERT(CGregorianCalendar::DaysInYear(1998) == 365);
  ASSERT(CGregorianCalendar::DaysInMonth(2000, January) == 31);
  ASSERT(CGregorianCalendar::DaysInMonth(2000, February) == 29);
  ASSERT(CGregorianCalendar::DaysInMonth(2001, February) == 28);
  ASSERT(CGregorianCalendar::DaysInMonth(2000, June) == 30);
  ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2000, January, 1) == 1);
  ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2001, January, 1) == 1);
  ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2000, December, 31) == 366);
  ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2001, December, 31) == 365);
  ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2000, July, 13) == 195);
  ASSERT(CGregorianCalendar::DaysSinceEndPreviousYear(2001, July, 13) == 194);
  ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2000, December, 31) == 365);
  ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2001, December, 31) == 364);
  ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2000, July, 13) == 194);
  ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2001, July, 13) == 193);
  ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2000, January, 1) == 0);
  ASSERT(CGregorianCalendar::DaysSinceFirstOfYear(2001, January, 1) == 0);

  
  //is it possible to have only 3 of a weekday in a month
  //Construction using the WeekOfMonth parameter
  x = CDate<CGregorianCalendar>(1995, September, 3, Friday);
  ASSERT(x == CDate<CGregorianCalendar>(1995, September, 15));
  x = CDate<CGregorianCalendar>(1995, September, 1, Saturday);
  ASSERT(x == CDate<CGregorianCalendar>(1995, September, 2));
  x = CDate<CGregorianCalendar>(1995, September, 1, Tuesday);
  ASSERT(x == CDate<CGregorianCalendar>(1995, September, 5));
  x = CDate<CGregorianCalendar>(1995, September, 5, Sunday);
  ASSERT(x == CDate<CGregorianCalendar>(1995, September, 24));
  x = CDate<CGregorianCalendar>(1995, September, 5, Friday);
  ASSERT(x == CDate<CGregorianCalendar>(1995, September, 29));



  //Test out the holidays
  CDate<> hol1 = CJulianGregorianHolidays<>::NewYearsDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::StValentinesDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::AshWednesday(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::StPatricksDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::GoodFriday(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::EasterSunday(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::CanadaDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::BastilleDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::ChristmasDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::MartinLutherKingDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::AbrahamLincolnsBirthday(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::GeorgeWashingtonsBirthday(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USPresidentsDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USIndependenceDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USArmedForcesDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USMemorialDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USFlagDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USLaborDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USColumbusDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USElectionDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USVeteransDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::USThanksgivingDay(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::PalmSunday(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::MaundyThursday(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::FeastOfTheAscension(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::FeastOfTheAnnunciation(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::ShroveTuesday(2006);
  s = hol1.Format();
  hol1 = CJulianGregorianHolidays<>::Pentecost(2006);
  s = hol1.Format();


  CDate<CMoslemCalendarCivil16> mdate4 = CMoslemNewYearFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format();
  mdate4 = CAshuraFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format();
  mdate4 = CNawlidAnNabiFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format();
  mdate4 = CLailatAlMiRajFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format();
  mdate4 = CLailatAlBaraaFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format();
  mdate4 = CRamdadanFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format();
  mdate4 = CLailatAlKadrFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format();
  mdate4 = CIdAlFitrFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format();
  mdate4 = CIdAlAdhaFunctor<CMoslemCalendarCivil16>()(1427);
  s = mdate4.Format();

  CArray<CDate<CGregorianCalendar>, CDate<CGregorianCalendar>& > occurences;
  MoslemOccurencesInGregorianYear<CIdAlAdhaFunctor<CMoslemCalendarCivil16>, CMoslemCalendarCivil16>(2006, occurences);
  _tprintf(_T("IdAlAdha in the Moslem Civil16 calendar occurences in the gregorian year 2006:\n"), s.operator LPCTSTR());
  int ii;
  for (ii=0; ii<occurences.GetSize(); ii++)
    _tprintf(_T("%s\n"), occurences.ElementAt(ii).Format());


  CDate<CHebrewCalendar> hdate1 = CRoshHaShanah1Functor()(5766); 
  s = hdate1.Format();
  s = hdate1.AsGregorianCDate().Format();
  hdate1 = CYomKippurFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CSukkotFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CHoshanahRabahFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CSheminiAzeretFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CCSimhatTorahOutsideIsraelFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CPesachFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CShviiPesachFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CHanukkah1Functor()(5766); 
  s = hdate1.Format();
  hdate1 = CHanukkah2Functor()(5766); 
  s = hdate1.Format();
  hdate1 = CHanukkah3Functor()(5766); 
  s = hdate1.Format();
  hdate1 = CHanukkah4Functor()(5766); 
  s = hdate1.Format();
  hdate1 = CHanukkah5Functor()(5766); 
  s = hdate1.Format();
  hdate1 = CHanukkah6Functor()(5766); 
  s = hdate1.Format();
  hdate1 = CHanukkah7Functor()(5766); 
  s = hdate1.Format();
  hdate1 = CHanukkah8Functor()(5766); 
  s = hdate1.Format();
  hdate1 = CPurimFunctor()(5766); 
  s = hdate1.Format();
  s = hdate1.AsGregorianCDate().Format();

  hdate1 = CTzomGedaliahFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CTzomTevetFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CTaanitEstherFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CTzomTammuzFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CTishahBeAvFunctor()(5766); 
  s = hdate1.Format();
  hdate1 = CShushanPurimFunctor()(5766); 
  s = hdate1.Format();
  long Year;
  for (Year=5760; Year <= 5800; Year++)
  {
    hdate1 = CYomHaShoahFunctor()(Year); 
    s = hdate1.Format();
  }
  for (Year=5760; Year<5800; Year++)
  {
    hdate1 = CYomHaZikkaronFunctor()(Year); 
    s = hdate1.Format();
  }

  hdate1 = CShavuotFunctor()(5766); 
  s = hdate1.Format();
  long OmerDayNumber;
  bool bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber);
  bOmer;
  hdate1 -= 3;
  s = hdate1.Format();
  bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber);
  hdate1 -= 45;
  s = hdate1.Format();
  bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber);
  hdate1 -= 1;
  s = hdate1.Format();
  bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber);
  hdate1 -= 1;
  s = hdate1.Format();
  bOmer = CHebrewHolidays::Omer(hdate1, OmerDayNumber);

  HewbrewOccurencesInGregorianYear<CPesachFunctor>(2006, occurences);
  _tprintf(_T("Pesach (Passover) in the Hebrew calendar occurences in the gregorian year 2006:\n"), s.operator LPCTSTR());
  for (ii=0; ii<occurences.GetSize(); ii++)
    _tprintf(_T("%s\n"), occurences.ElementAt(ii).Format().operator LPCTSTR());

  HewbrewOccurencesInGregorianYear<CTzomTevetFunctor>(1982, occurences);
  _tprintf(_T("Tzom Tevet in the Hebrew calendar occurences in the gregorian year 1982:\n"), s.operator LPCTSTR());
  for (ii=0; ii<occurences.GetSize(); ii++)
    _tprintf(_T("%s\n"), occurences.ElementAt(ii).Format());
  HewbrewOccurencesInGregorianYear<CTzomTevetFunctor>(1984, occurences);
  _tprintf(_T("Tzom Tevet in the Hebrew calendar occurences in the gregorian year 1984:\n"), s.operator LPCTSTR());
  for (ii=0; ii<occurences.GetSize(); ii++)
    _tprintf(_T("%s\n"), occurences.ElementAt(ii).Format());




  long lYear = CDate<CGregorianCalendar>::CurrentLocalDate().GetYear();
  WORD wMonth = 0, wDay = 0;
  CGregorianCalendar::EasterSunday(lYear, wMonth, wDay);
  x = CDate<CGregorianCalendar>(lYear, wMonth, wDay);
  s = x.Format();
  _tprintf(_T("Easter Sunday occurs on %s this Gregorian year\n"), s.operator LPCTSTR());

  CJulianCalendar::EasterSunday(lYear, wMonth, wDay);
  p = CDate<CJulianCalendar>(lYear, wMonth, wDay);
  s = p.Format();
  _tprintf(_T("Easter Sunday occurs on %s this Julian year\n"), s.operator LPCTSTR());


  //Test out the hebrew calendar support
  CDate<CJulianCalendar> hepoch(-3760, October, 7);


  hdate1 = CDate<CHebrewCalendar>(4682, Nissan, 15); 
  s = hdate1.AsJulianCDate().Format();
  DateS ds2 = hdate1.GetDate();
  ds2;
  double Molah = CHebrewCalendar::Molad(4682, Tishrei);
  CLDate<CJulianCalendar> moladD(Molah, EPOCH_GREG);
  s = moladD.Format();
  double Molah2 = CHebrewCalendar::Molad(4683, Tishrei);
  CLDate<CJulianCalendar> moladD2(Molah2, EPOCH_GREG);
  s = moladD2.Format();
  
  long HebrewNewYear = CHebrewCalendar::HebrewNewYear(4682);
  HebrewNewYear;
  WORD wDelay = CHebrewCalendar::HebrewNewYearDelay(4682);
  wDelay;
  _tprintf(_T("There are %d days in the Hebrew year 4682 A.M.\n"), CHebrewCalendar::DaysInYear(4682));
  if (CHebrewCalendar::IsLeap(4682))
    _tprintf(_T("Hebrew year 4682 A.M is a leap year.\n"));
  else
    _tprintf(_T("Hebrew year 4682 A.M is a not leap year.\n"));
  long nHebrewElapsedDays = CHebrewCalendar::HebrewCalendarElapsedDays(4682);
  nHebrewElapsedDays;
  bool bLongCheshvan = CHebrewCalendar::LongCheshvan(4682);
  if (bLongCheshvan)
    _tprintf(_T("Cheshvan is 30 days in length in the Hebrew year 4682 A.M\n"));
  else
    _tprintf(_T("Cheshvan is 29 days in length inin the Hebrew year 4682 A.M\n"));  
  bool bShortCheshvan = CHebrewCalendar::ShortCheshvan(4682);
  bShortCheshvan;
  bool bLongKislev = CHebrewCalendar::LongKislev(4682);
  if (bLongKislev)
    _tprintf(_T("Kislev is 30 days in length in the Hebrew year 4682 A.M\n"));
  else
    _tprintf(_T("Kislev is 30 days in length in the Hebrew year 4682 A.M\n"));
  bool bShortKislev = CHebrewCalendar::ShortKislev(4682);
  bShortKislev;
  _tprintf(_T("Hebrew Months for the year 4682 A.M.\n"));
  WORD i;
  for (i=Nissan; i<=CHebrewCalendar::LastMonthOfYear(CHebrewCalendar::IsLeap(4682)); i++)
    _tprintf(_T("Month : %s %s %d\n"), CHebrewCalendar::GetFullStringMonth(i, LOCALE_USER_DEFAULT).operator LPCTSTR(), CHebrewCalendar::GetAbrStringMonth(i, LOCALE_USER_DEFAULT).operator LPCTSTR(), CHebrewCalendar::DaysInMonth(4682, i));
  _tprintf(_T("Hebrew days of the week\n"));
  for (i=Sunday; i<=Saturday; i++)
    _tprintf(_T("Week Day: %s %s %s\n"), CHebrewCalendar::GetFullStringDayOfWeek((DayOfWeek) i, LOCALE_USER_DEFAULT).operator LPCTSTR(), CHebrewCalendar::GetAbrStringDayOfWeek((DayOfWeek) i, LOCALE_USER_DEFAULT).operator LPCTSTR(), CJulianGregorianCalendar<CGregorianLeapFunctor>::GetFullStringDayOfWeek((DayOfWeek) i, LOCALE_USER_DEFAULT));
  
  hdate1 = CDate<CHebrewCalendar>::CurrentLocalDate();
  s = hdate1.Format();
  _tprintf(_T("The current local date (in the Hebrew Calendar) is %s\n"), s.operator LPCTSTR());

  s = CDate<>(2006, March, 22).AsHebrewCDate().Format();


  
  //test the range of the class

  long StartYear = -100000;
  long EndYear = 100000;

  //Uncomment if you want to perform a full range check on CDate's construction routines
  ///*
  _tprintf(_T("\nNow going to construct a CDate for all dates\n"));
  _tprintf(_T("between the years %ld and %ld\n"), StartYear, EndYear);
  _tprintf(_T("This may take a while\n"));
  _tprintf(_T("<Hit Enter to continue>\n"));
  chr = getchar();

  _tprintf(_T("Doing Julian Calendar validation\n"));
  DWORD StartTime = GetTickCount();
  long nLastDateValue = 0;
  bool bLastDateValueValid = false;

  for (lYear = StartYear; lYear < EndYear; lYear++)
  {
    if (lYear % 1000 == 0)
      _tprintf(_T("Testing Year %ld\n"), lYear);
    for (WORD wMonth = January; wMonth <= December; wMonth++)
    {
      for (wDay = 1; wDay <= CJulianCalendar::DaysInMonth(lYear, wMonth); wDay++)
      {
        x2 = CDate<CJulianCalendar>(lYear, wMonth, wDay);
        if (x2.IsValid())
        {
          DateS ds = x2.GetDate();
          if ((ds.lYear != lYear) || (ds.wMonth != wMonth) || (ds.wDay != wDay))
          {
            s = x2.Format();
            _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (x2.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = x2.GetValue();
        }
        else
          ASSERT(false);
      }
    }
  }
  DWORD EndTime = GetTickCount();
  DWORD MilliSecondsTaken = (EndTime - StartTime);
  _tprintf(_T("Julian Speed: About %f milliseconds for 1000 years\n"), (MilliSecondsTaken*1000.0)/(EndYear - StartYear));

  _tprintf(_T("Doing Gregorian Calendar validation\n"));
  StartTime = GetTickCount();
  bLastDateValueValid = false;
  for (lYear = StartYear; lYear < EndYear; lYear++)
  {
    if (lYear % 1000 == 0)
      _tprintf(_T("Testing Year %ld\n"), lYear);
    for (WORD wMonth = January; wMonth <= December; wMonth++)
    {
      WORD wDay;
      for (wDay = 1; wDay <= CGregorianCalendar::DaysInMonth(lYear, wMonth); wDay++)
      {
        x = CDate<CGregorianCalendar>(lYear, wMonth, wDay);
        if (x.IsValid())
        {
          DateS ds = x.GetDate();
          if ((ds.lYear != lYear) || (ds.wMonth != wMonth) || (ds.wDay != wDay))
          {
            _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (x.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = x.GetValue();
        }
        else
          ASSERT(false);
      }
    }
  }
  EndTime = GetTickCount();
  MilliSecondsTaken = (EndTime - StartTime);
  _tprintf(_T("Gregorian Speed: About %f milliseconds for 1000 years\n"), (MilliSecondsTaken*1000.0)/(EndYear - StartYear));

  _tprintf(_T("\nDoing Moslem Civil 16 Calendar validation\n"));
  StartTime = GetTickCount();
  bLastDateValueValid = false;
  for (lYear = StartYear; lYear < EndYear; lYear++)
  {
    if (lYear % 1000 == 0)
      _tprintf(_T("Testing Year %ld\n"), lYear);
    for (WORD wMonth = Muharram; wMonth <= Dhu_l_Hijja; wMonth++)
    {
      WORD wDay;
      for (wDay = 1; wDay <= CMoslemCalendarCivil16::DaysInMonth(lYear, wMonth); wDay++)
      {
        mdate4 = CDate<CMoslemCalendarCivil16>(lYear, wMonth, wDay);
        if (mdate4.IsValid())
        {
          DateS ds = mdate4.GetDate();
          if ((ds.lYear != lYear) || (ds.wMonth != wMonth) || (ds.wDay != wDay))
          {
            _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (mdate4.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = mdate4.GetValue();
        }
        else
          ASSERT(false);
      }
    }
  }
  EndTime = GetTickCount();
  MilliSecondsTaken = (EndTime - StartTime);
  _tprintf(_T("Moslem Civil 16 Speed: About %f milliseconds for 1000 years\n"), (MilliSecondsTaken*1000.0)/(EndYear - StartYear));

  _tprintf(_T("\nDoing Moslem Civil 15 Calendar validation\n"));
  CDate<CMoslemCalendarCivil15> mdate1;
  StartTime = GetTickCount();
  bLastDateValueValid = false;
  for (lYear = StartYear; lYear < EndYear; lYear++)
  {
    if (lYear % 1000 == 0)
      _tprintf(_T("Testing Year %ld\n"), lYear);
    for (WORD wMonth = Muharram; wMonth <= Dhu_l_Hijja; wMonth++)
    {
      WORD wDay;
      for (wDay = 1; wDay <= CMoslemCalendarCivil15::DaysInMonth(lYear, wMonth); wDay++)
      {
        mdate1 = CDate<CMoslemCalendarCivil15>(lYear, wMonth, wDay);
        if (mdate1.IsValid())
        {
          DateS ds = mdate1.GetDate();
          if ((ds.lYear != lYear) || (ds.wMonth != wMonth) || (ds.wDay != wDay))
          {
            _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (mdate1.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = mdate1.GetValue();
        }
        else
          ASSERT(false);
      }
    }
  }
  EndTime = GetTickCount();
  MilliSecondsTaken = (EndTime - StartTime);
  _tprintf(_T("Moslem Civil 15 Speed: About %f milliseconds for 1000 years\n"), (MilliSecondsTaken*1000.0)/(EndYear - StartYear));

  _tprintf(_T("\nDoing Moslem Astronomical 15 Calendar validation\n"));
  CDate<CMoslemCalendarAstronomical15> mdate5;
  StartTime = GetTickCount();
  bLastDateValueValid = false;
  for (lYear = StartYear; lYear < EndYear; lYear++)
  {
    if (lYear % 1000 == 0)
      _tprintf(_T("Testing Year %ld\n"), lYear);
    for (WORD wMonth = Muharram; wMonth <= Dhu_l_Hijja; wMonth++)
    {
      WORD wDay;
      for (wDay = 1; wDay <= CMoslemCalendarAstronomical15::DaysInMonth(lYear, wMonth); wDay++)
      {
        mdate5 = CDate<CMoslemCalendarAstronomical15>(lYear, wMonth, wDay);
        if (mdate5.IsValid())
        {
          DateS ds = mdate5.GetDate();
          if ((ds.lYear != lYear) || (ds.wMonth != wMonth) || (ds.wDay != wDay))
          {
            _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (mdate5.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = mdate5.GetValue();
        }
        else
          ASSERT(false);
      }
    }
  }
  EndTime = GetTickCount();
  MilliSecondsTaken = (EndTime - StartTime);
  _tprintf(_T("Moslem Astronomical 15 Speed: About %f milliseconds for 1000 years\n"), (MilliSecondsTaken*1000.0)/(EndYear - StartYear));

  _tprintf(_T("\nDoing Moslem Astronomical 16 Calendar validation\n"));
  CDate<CMoslemCalendarAstronomical16> mdate8;
  StartTime = GetTickCount();
  bLastDateValueValid = false;
  for (lYear = StartYear; lYear < EndYear; lYear++)
  {
    if (lYear % 1000 == 0)
      _tprintf(_T("Testing Year %ld\n"), lYear);
    for (WORD wMonth = Muharram; wMonth <= Dhu_l_Hijja; wMonth++)
    {
      WORD wDay;
      for (wDay = 1; wDay <= CMoslemCalendarAstronomical16::DaysInMonth(lYear, wMonth); wDay++)
      {
        mdate8 = CDate<CMoslemCalendarAstronomical16>(lYear, wMonth, wDay);
        if (mdate8.IsValid())
        {
          DateS ds = mdate8.GetDate();
          if ((ds.lYear != lYear) || (ds.wMonth != wMonth) || (ds.wDay != wDay))
          {
            _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (mdate8.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = mdate8.GetValue();
        }
        else
          ASSERT(false);
      }
    }
  }
  EndTime = GetTickCount();
  MilliSecondsTaken = (EndTime - StartTime);
  _tprintf(_T("Moslem Astronomical 16 Speed : About %f milliseconds for 1000 years\n"), (MilliSecondsTaken*1000.0)/(EndYear - StartYear));

  _tprintf(_T("\nDoing Hebrew Calendar validation\n"));
  StartTime = GetTickCount();
  bLastDateValueValid = false;
  for (lYear = StartYear; lYear < EndYear; lYear++)
  {
    if (lYear % 1000 == 0)
      _tprintf(_T("Testing Year %ld\n"), lYear);

    //Note for testing purpose we need to use a different ordering for the Hebrew months, because the New Year occurs in the 
    //month Tishri which is logically the 7th Month as opposed the the first month which Nisan
    WORD wMonth;
    for (wMonth = Tishrei; wMonth <= CHebrewCalendar::LastMonthOfYear(CHebrewCalendar::IsLeap(lYear)); wMonth++)
    {
      WORD wDay;
      for (wDay = 1; wDay <= CHebrewCalendar::DaysInMonth(lYear, wMonth); wDay++)
      {
        hdate1 = CDate<CHebrewCalendar>(lYear, wMonth, wDay);
        if (hdate1.IsValid())
        {
          DateS ds = hdate1.GetDate();
          if ((ds.lYear != lYear) || (ds.wMonth != wMonth) || (ds.wDay != wDay))
          {
            _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (hdate1.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = hdate1.GetValue();
        }
        else
          ASSERT(false);
      }
    }
    for (wMonth = Nissan; wMonth <= Elul; wMonth++)
    {
      WORD wDay;
      for (wDay = 1; wDay <= CHebrewCalendar::DaysInMonth(lYear, wMonth); wDay++)
      {
        hdate1 = CDate<CHebrewCalendar>(lYear, wMonth, wDay);
        if (hdate1.IsValid())
        {
          DateS ds = hdate1.GetDate();
          if ((ds.lYear != lYear) || (ds.wMonth != wMonth) || (ds.wDay != wDay))
          {
            _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
            ASSERT(false);
          }
          if (bLastDateValueValid)
          {
            if (hdate1.GetValue() != (nLastDateValue + 1))
            {
              _tprintf(_T("Error occured testing %d/%d/%d\n"), lYear, static_cast<int>(wMonth), static_cast<int>(wDay));
              ASSERT(false);
            }
          }
          bLastDateValueValid = true;
          nLastDateValue = hdate1.GetValue();
        }
        else
          ASSERT(false);
      }
    }
  }
  EndTime = GetTickCount();
  MilliSecondsTaken = (EndTime - StartTime);
  _tprintf(_T("hebrew Speed: About %f milliseconds for 1000 years\n"), (MilliSecondsTaken*1000.0)/(EndYear - StartYear));

  _tprintf(_T("<Hit Enter to continue>\n"));
  chr = getchar();
  //*/
  

  //Try out the GetAsTM accessor
  x = CDate<CGregorianCalendar>::Epoch2000();
  tm tms;
  bool bSuccess = x.GetAsTM(tms);

  //Try out the GetAsSystemTime accessor
  bSuccess = x.GetAsSystemTime(st);

  //Try out the GetAsFileTime accessor
  bSuccess = x.GetAsFileTime(ft);
      
  // Try out the other operators
  long l = 0;
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  l = x.GetValue();
  _tprintf(_T("Today is %ld days since the Gregorian epoch\n"), l);
  l = x.JD();
  _tprintf(_T("Today is %ld days since the Julian Day epoch\n"), l);
  if (x.IsLeap())
    _tprintf(_T("This year is a leap year\n"));
  else
    _tprintf(_T("This year is not a leap year\n"));
  WORD diy = x.DaysInYear();
  _tprintf(_T("This year contains %d days \n"), diy);
  WORD dim = x.DaysInMonth();
  _tprintf(_T("This month contains %d days \n"), dim);
  x.AddYear(100);
  s = x.Format();
  _tprintf(_T("100 years from today is %s\n"), s.operator LPCTSTR());
  x.AddYear(-100);
  s = x.Format();
  _tprintf(_T("100 years from today - 100 years is %s\n"), s.operator LPCTSTR());
  x.AddMonth(100);
  s = x.Format();
  _tprintf(_T("100 months from today is %s\n"), s.operator LPCTSTR());
  x.AddMonth(-100);
  s = x.Format();
  _tprintf(_T("100 months from today - 100 months is %s\n"), s.operator LPCTSTR());
  x.AddWeek(100);
  s = x.Format();
  _tprintf(_T("100 weeks from today is %s\n"), s.operator LPCTSTR());
  x.AddWeek(-100);
  s = x.Format();
  _tprintf(_T("100 weeks from today - 100 weeks is %s\n"), s.operator LPCTSTR());
  dow = x.GetDayOfWeek();
  _tprintf(_T("Todays day of week is %d\n"), dow);
  s = x.GetFullStringDayOfWeek(LOCALE_USER_DEFAULT);
  _tprintf(_T("Todays full string day of week is %s\n"), s.operator LPCTSTR());
  s = x.GetAbrStringDayOfWeek(LOCALE_USER_DEFAULT);
  _tprintf(_T("Todays Abreviated string day of week is %s\n"), s.operator LPCTSTR());
  s = x.GetFullStringMonth(LOCALE_USER_DEFAULT);
  _tprintf(_T("Todays full string Month is %s\n"), s.operator LPCTSTR());
  s = x.GetAbrStringMonth(LOCALE_USER_DEFAULT);
  _tprintf(_T("Todays Abreviated string Month is %s\n"), s.operator LPCTSTR());
  _tprintf(_T("Today is %d days since the 1st of this year\n"), x.DaysSinceFirstOfYear());
  _tprintf(_T("Today is %d days since the 0th of this year\n"), x.DaysSinceEndPreviousYear());

  bSuccess = x.GetAsSystemTime(st);

  bool IsCE;
  long y = x.GetCEBCEYear(IsCE);
  y;

  //Creation of other CDate's from this instance
  x = CDate<CGregorianCalendar>(1995, February, 3);
  s = x.Format();
  _tprintf(_T("The CDate used to test CDate creation from this instance is %s\n"), s.operator LPCTSTR());
  CDate<CGregorianCalendar> c = x.FirstThisMonth();
  s = c.Format();
  _tprintf(_T("FirstThisMonth returns %s\n"), s.operator LPCTSTR());
  c = x.LastThisMonth();
  s = c.Format();
  _tprintf(_T("LastThisMonth returns %s\n"), s.operator LPCTSTR());
  c = x.FirstThisYear();
  s = c.Format();
  _tprintf(_T("FirstThisYear returns %s\n"), s.operator LPCTSTR());
  c = x.LastThisYear();
  s = c.Format();
  _tprintf(_T("LastThisYear returns %s\n"), s.operator LPCTSTR());

  //assigment operator has been tested before
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  c = x + 100;
  s = c.Format();
  _tprintf(_T("100 days from today is %s\n"), s.operator LPCTSTR());
  c = c - 100;
  s = c.Format();
  _tprintf(_T("100 days from today  - 100 days is %s\n"), s.operator LPCTSTR());
  long diff = CDate<CGregorianCalendar>(2000, January, 1) - CDate<CGregorianCalendar>::CurrentLocalDate();
  _tprintf(_T("the timespan between Epoch 2000 and today is %ld\n"), diff);
  c += 1000;
  s = c.Format();
  _tprintf(_T("today incremented by 1000 days is %s\n"), s.operator LPCTSTR());
  c -= 1000;
  s = c.Format();
  _tprintf(_T("today incremented by 1000 days decremented by 1000 is %s\n"), s.operator LPCTSTR());
  ++c;
  s = c.Format();
  _tprintf(_T("Tomorrow is %s\n"), s.operator LPCTSTR());
  --c;
  --c;
  s = c.Format();
  _tprintf(_T("Yesterday was %s\n"), s.operator LPCTSTR());

  c = CDate<CGregorianCalendar>::CurrentLocalDate();
  
  //Full test of the Format function
  _tprintf(_T("A full test of Format for Today gives\n"));
  s = c.Format(_T("%a"));
  _tprintf(_T("Abbreviated weekday name : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%A"));
  _tprintf(_T("Full weekday name : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%b"));
  _tprintf(_T("Abbreviated month name : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%B"));
  _tprintf(_T("Full month name : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%d"));
  _tprintf(_T("Day of month as decimal number (01 - 31) : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%j"));
  _tprintf(_T("Day of year as decimal number (001 - 366) : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%m"));
  _tprintf(_T("Month as decimal number (01 - 12) : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%U"));
  _tprintf(_T("Week of year as decimal number : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%w"));
  _tprintf(_T("Weekday as decimal number (1 - 7), Sunday is 1 : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%x"));
  _tprintf(_T("Date representation for current locale : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%y"));
  _tprintf(_T("Year without century, as decimal number (00 - 99) : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%Y"));
  _tprintf(_T("Year with century, as decimal number : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%#x"));
  _tprintf(_T("Long date representation, appropriate to current locale : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%#d"));
  _tprintf(_T("Day of month without leading zeros : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%#j"));
  _tprintf(_T("Day of year without leading zeros : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%#m"));
  _tprintf(_T("Month without leading zeroes : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%#U"));
  _tprintf(_T("Week of year without leading zeroes : %s\n"), s.operator LPCTSTR());
  s = c.Format(_T("%#y"));
  _tprintf(_T("Year without century without leading zeroes : %s\n"), s.operator LPCTSTR());

  // Try out the week of year function
  x = CDate<CGregorianCalendar>(1997, January, 1);
  s = x.Format();
  _tprintf(_T("The first of January for testing the Week Of Year function is %s\n"), s.operator LPCTSTR());
  //_tprintf(_T("Setting the begining of the week to Thursday\n"));
  CDate<CGregorianCalendar> v = CDate<CGregorianCalendar>(1997, January, 1, Thursday);
  s = v.Format();
  _tprintf(_T("The 1st Thursday of 1997 is %s\n"), s.operator LPCTSTR());
  for (i=1; i<32; i++)
  {
    CDate<CGregorianCalendar> p(1997, January, i);
    WORD woy = p.GetWeekOfYear(LOCALE_USER_DEFAULT);
    s = p.Format();
    _tprintf(_T("The week of the year for %s is"), s);
    _tprintf(_T(" %d\n"), woy);
  }

  // Try out the week of year function
  x = CDate<CGregorianCalendar>(1997, January, 1);
  s = x.Format();
  _tprintf(_T("The first of January for testing the Week Of year function is %s\n"), s.operator LPCTSTR());
  v = CDate<CGregorianCalendar>(1997, January, 1, Monday);
  s = v.Format();
  _tprintf(_T("The 1st Monday of 1997 is %s\n"), s.operator LPCTSTR());
  for (i=1; i<32; i++)
  {
    CDate<CGregorianCalendar> p(x.GetYear(), x.GetMonth(), i);
    WORD woy = p.GetWeekOfYear(LOCALE_USER_DEFAULT);
    s = p.Format();
    _tprintf(_T("The week of the year for %s is"), s.operator LPCTSTR());
    _tprintf(_T(" %d\n"), woy);
  }

  // Try out the week of month function
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  x = x.FirstThisMonth();
  s = x.Format();
  _tprintf(_T("The first of this month for testing the Week Of month function is %s\n"), s.operator LPCTSTR());
  v = CDate<CGregorianCalendar>(x.GetYear(), x.GetMonth(), 1, Thursday);
  s = v.Format();
  _tprintf(_T("The 1st Thursday of the month is %s\n"), s.operator LPCTSTR());
  for (i=1; i<=x.DaysInMonth(); i++)
  {
    CDate<CGregorianCalendar> p(x.GetYear(), x.GetMonth(), i);
    WORD wom = p.GetWeekOfMonth(LOCALE_USER_DEFAULT);
    s = p.Format();
    _tprintf(_T("The week of the month for %s is"), s.operator LPCTSTR());
    _tprintf(_T(" %d\n"), wom);
  }

  //try out the collate function
  x = CDate<CGregorianCalendar>::CurrentLocalDate();
  long coll = x.Collate();
  _tprintf(_T("The collate function for today returns %ld\n"), coll);
  x.SetFromCollate(coll);
  _tprintf(_T("Using SetFromCollate for today created %s\n"), x.Format().operator LPCTSTR());

  //try out the equality operators
  x = CDate<CGregorianCalendar>::CurrentSystemDate();
  CDate<CGregorianCalendar> y2 = x;
  ASSERT(x == y2);
  ASSERT(!(x != y2));
  y2 += 1;
  ASSERT(y2 != x);
  ASSERT(y2 >= x);
  ASSERT(x <= y2);
  ASSERT(x < y2);

  //mdate1 = CDate<CMoslemCalendar>(1421, Muharram, 1);
  //s = mdate1.Format();
  //CDate<CJulianCalendar> mdate2 = mdate1.AsJulianCDate();
  //s = mdate2.Format();
  //CDate<CGregorianCalendar> mdate3 = mdate1.AsGregorianCDate(); 
  //s = mdate3.Format();

  //x = CDate<CGregorianCalendar>(1991, August, 13);
  //mdate1 = x.AsMoslemCDate();
  //s = mdate1.Format();

  //mdate1 = CDate<CMoslemCalendar>(1427, Dhu_l_Hijja, 28);
  //DateS ds = mdate1.GetDate();
  //ds = mdate1.AsGregorianCDate().GetDate();
  //ds = mdate1.AsJulianCDate().GetDate();

  p = CDate<CJulianCalendar>(622, July, 16);
  mdate1 = CDate<CMoslemCalendarCivil15>(1, 1, 1);
  s = mdate1.Format();
  DateS ds = mdate1.AsJulianCDate().GetDate();

  CDate<CMoslemCalendarCivil16> mdate6(16, 6, 5);
  s = mdate6.Format();
  ds = mdate6.AsJulianCDate().GetDate();

  mdate1 = CDate<CMoslemCalendarCivil15>(16, 6, 5);
  s = mdate1.Format();
  ds = mdate1.AsJulianCDate().GetDate();

  CDate<CMoslemCalendarAstronomical16> mdate7(16, 6, 5);
  s = mdate7.Format();
  ds = mdate7.AsJulianCDate().GetDate();

  CDate<CMoslemCalendarAstronomical16> mdate9(16, 6, 5);
  s = mdate9.Format();
  ds = mdate9.AsJulianCDate().GetDate();

  ds = mdate1.GetDate();
  ds = mdate1.AsGregorianCDate().GetDate();
  ds = mdate1.AsJulianCDate().GetDate();

  mdate1 = CDate<CMoslemCalendarCivil15>::CurrentLocalDate();
  _tprintf(_T("A full test of Format for Today in the moslem Civil 15 calendar gives\n"));
  s = mdate1.Format(_T("%a"));
  _tprintf(_T("Abbreviated weekday name : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%A"));
  _tprintf(_T("Full weekday name : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%b"));
  _tprintf(_T("Abbreviated month name : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%B"));
  _tprintf(_T("Full month name : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%d"));
  _tprintf(_T("Day of month as decimal number (01 - 31) : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%j"));
  _tprintf(_T("Day of year as decimal number (001 - 355) : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%m"));
  _tprintf(_T("Month as decimal number (01 - 12) : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%U"));
  _tprintf(_T("Week of year as decimal number : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%w"));
  _tprintf(_T("Weekday as decimal number (1 - 7), Sunday is 1 : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%x"));
  _tprintf(_T("Date representation for current locale : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%y"));
  _tprintf(_T("Year without century, as decimal number (00 - 99) : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%Y"));
  _tprintf(_T("Year with century, as decimal number : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%#x"));
  _tprintf(_T("Long date representation, appropriate to current locale : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%#d"));
  _tprintf(_T("Day of month without leading zeros : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%#j"));
  _tprintf(_T("Day of year without leading zeros : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%#m"));
  _tprintf(_T("Month without leading zeroes : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%#U"));
  _tprintf(_T("Week of year without leading zeroes : %s\n"), s.operator LPCTSTR());
  s = mdate1.Format(_T("%#y"));
  _tprintf(_T("Year without century without leading zeroes : %s\n"), s.operator LPCTSTR());

  //Test of the CLTimeSpan class
  
  //first the constructors
  CLTimeSpan ts;
  ASSERT(!ts.IsValid());

  ts = CLTimeSpan(3, 4, 5, 6);
  long dd = ts.GetTotalDays();
  WORD hh = ts.GetHours();
  WORD mm = ts.GetMinutes();
  WORD ss = ts.GetSeconds();
  s = ts.Format();

  ts = CLTimeSpan(3, 27, 59, 69);
  s = ts.Format();

  ts = CLTimeSpan(-3, 4, 5, 6);
  s = ts.Format();

  CLTimeSpan ts2(ts);
  s = ts2.Format();

  CTimeSpan OneDay(1, 0, 0, 0);
  ts = CLTimeSpan(OneDay);
  s = ts.Format();

  ts = CLTimeSpan(86400);
  s = ts.Format();

  ts = CLTimeSpan(-23);
  dd = ts.GetTotalDays();
  hh = ts.GetHours();
  mm = ts.GetMinutes();
  ss = ts.GetSeconds();
  s = ts.Format();
  double secs = ts.SecondsAsDouble();

  ts = CLTimeSpan(-86401);
  dd = ts.GetTotalDays();
  hh = ts.GetHours();
  mm = ts.GetMinutes();
  ss = ts.GetSeconds();
  s = ts.Format();
  secs = ts.SecondsAsDouble();

  ts = CLTimeSpan(-86401);
  dd = ts.GetTotalDays();
  hh = ts.GetHours();
  mm = ts.GetMinutes();
  ss = ts.GetSeconds();
  s = ts.Format();
  secs = ts.SecondsAsDouble();
  
  ts = CLTimeSpan(86399);
  s = ts.Format();
  secs = ts.SecondsAsDouble();

  //COleDateTimeSpan Construction
  COleDateTimeSpan olets(19, 12, 10, 3);
  ts = CLTimeSpan(olets);  
  s = ts.Format();

  //The __int64 constructor
  ts = CLTimeSpan(3155760000);
  dd = ts.GetTotalDays();
  hh = ts.GetHours();
  mm = ts.GetMinutes();
  ss = ts.GetSeconds();
  s = ts.Format();

  //Set operators have been tested via the constructors

  //static Constructors

  ts = CLTimeSpan::OneDay();
  s = ts.Format();
  _tprintf(_T("One Day as a time span is %s\n"), s.operator LPCTSTR());
  ts = CLTimeSpan::OneHour();
  s = ts.Format();
  _tprintf(_T("One Hour as a time span is %s\n"), s.operator LPCTSTR());
  ts = CLTimeSpan::OneMinute();
  s = ts.Format();
  _tprintf(_T("One Minute as a time span is %s\n"), s.operator LPCTSTR());
  ts = CLTimeSpan::OneSecond();
  s = ts.Format();
  _tprintf(_T("One Second as a time span is %s\n"), s.operator LPCTSTR());

  //days, Hours, Minutes & seconds accessors
  //already tested via Format

  //Overloaded Arithmetic Operators
  CLTimeSpan ts1(CLTimeSpan(365, 0, 0, 0));
  ts2 = CLTimeSpan::OneDay();
  //operator= already tested           //not fully tested yet
  CLTimeSpan ts3 = ts1 + ts2;
  _tprintf(_T("365 days + One day is %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = ts1 - ts2;
  _tprintf(_T("365 days - One day is %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = ts2 - ts1;

  _tprintf(_T("One day - One year is %s\n"), ts3.Format().operator LPCTSTR());
  ts1 = CLTimeSpan::CLTimeSpan(365, 0, 0, 0);
  ts2 = CLTimeSpan::OneDay();
  ts1 += ts2;
  _tprintf(_T("365 days incremented by one day is %s\n"), ts1.Format().operator LPCTSTR());
  ts1 -= ts2;
  _tprintf(_T("365 days + one day decremented by one day is %s\n"), ts1.Format().operator LPCTSTR());
  ts1 = CLTimeSpan(365, 0, 0, 0);
  ts2 -= ts1;
  _tprintf(_T("One day decremented by 365 days is %s\n"), ts2.Format().operator LPCTSTR());
  ts1 = CLTimeSpan(365, 0, 0, 0);
  ts1.Negate();
  ts2 = CLTimeSpan::OneDay();
  ts3 = ts1 - ts2;
  _tprintf(_T("-365 days - one day is %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = ts1 + ts2;
  _tprintf(_T("-365 days + one day is %s\n"), ts3.Format().operator LPCTSTR());
  ts1 = CLTimeSpan(365, 0, 0, 0);
  ts2 = CLTimeSpan::OneDay();
  ts2.Negate();
  ts3 = ts1 - ts2;
  _tprintf(_T("365 days - (-one day) is %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = ts1 + ts2;
  _tprintf(_T("365 days + (-one day) is %s\n"), ts3.Format().operator LPCTSTR());
  ts1 = CLTimeSpan(365, 0, 0, 0);
  ts1.Negate();
  ts2 = CLTimeSpan::OneDay();
  ts1 += ts2;
  _tprintf(_T("-365 days incremented by one day is %s\n"), ts1.Format().operator LPCTSTR());
  ts1 -= ts2;
  _tprintf(_T("-365 days + 1 day decremented by one day is %s\n"), ts1.Format().operator LPCTSTR());
  ts2 = CLTimeSpan(365, 0, 0, 0);
  ts2.Negate();
  ts1 -= ts2;
  _tprintf(_T("-365 days decremented by -one year is %s\n"), ts1.Format().operator LPCTSTR());
  ts1 += ts2;
  _tprintf(_T("-365 days - -365 days incremented by -365 days is %s\n"), ts1.Format().operator LPCTSTR());
  ts1 = CLTimeSpan(365, 0, 0, 0);
  ts2 = CLTimeSpan::OneDay();
  ts2.Negate();
  ts3 = ts1 - ts2;
  _tprintf(_T("365 days - -one day is %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = ts1 + ts2;
  _tprintf(_T("365 days + -one day is %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = ts2 - ts1;
  _tprintf(_T("-One day - 365 days is %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = ts2 + ts1;
  _tprintf(_T("-One day + 365 days is %s\n"), ts3.Format().operator LPCTSTR());
  ts1 = CLTimeSpan(365, 0, 0, 0);
  ts2 = CLTimeSpan::OneDay();
  ts3 = ts1 * 100;
  
  dd = ts3.GetTotalDays();
  hh = ts3.GetHours();
  mm = ts3.GetMinutes();
  ss = ts3.GetSeconds();
  s = ts3.Format();
  
  _tprintf(_T("365 days * 100 is %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = 100 * ts1;
  _tprintf(_T("100 * 365 days is %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = ts1 / 3;
  _tprintf(_T("365 days / 3 is %s\n"), ts3.Format().operator LPCTSTR());
  ts1 *= 100;
  _tprintf(_T("365 days *= 100 is %s\n"), ts1.Format().operator LPCTSTR());
  ts1 = CLTimeSpan(365, 0, 0, 0);
  ts1 /= 100;
  _tprintf(_T("365 days /= 100 is %s\n"), ts1.Format().operator LPCTSTR());
  ts1 = CLTimeSpan(365, 6, 10, 34);
  ts3 = ts1 * 100;
  _tprintf(_T("%s * 100 is"), ts1.Format().operator LPCTSTR());
  _tprintf(_T(" %s\n"), ts3.Format().operator LPCTSTR());
  ts3 = ts1 / 100;
  _tprintf(_T("%s / 100 is"), ts1.Format().operator LPCTSTR());
  _tprintf(_T(" %s\n"), ts3.Format().operator LPCTSTR());

  s = ts1.Format();
  ts3 = - ts1;
  s = ts3.Format();

  //overloaded equality operators
  ts1 = CLTimeSpan(365, 0, 0, 0);
  ts2 = CLTimeSpan::OneDay();
  ASSERT(ts1 >= ts2);
  ASSERT(ts1 > ts2);
  ASSERT(ts2 < ts1);
  ASSERT(ts2 <= ts1);
  ASSERT(ts2 != ts1);
  ASSERT(!(ts2 == ts1));
  
  //Full test of the Format function
  ts1 = CLTimeSpan(1, 2, 3, 4);
  _tprintf(_T("A full test of CLTimeSpan::Format for %s gives\n"), ts1.Format().operator LPCTSTR());
  s = ts1.Format(_T("%D"));
  _tprintf(_T("Days in the timespan : %s\n"), s.operator LPCTSTR());
  s = ts1.Format(_T("%H"));
  _tprintf(_T("Hours in the timespan : %s\n"), s.operator LPCTSTR());
  s = ts1.Format(_T("%M"));
  _tprintf(_T("Minutes in the timespan : %s\n"), s.operator LPCTSTR());
  s = ts1.Format(_T("%S"));
  _tprintf(_T("Seconds in the timespan : %s\n"), s.operator LPCTSTR());
  s = ts1.Format(_T("%#H"));
  _tprintf(_T("Hours in the timespan without leading zeros: %s\n"), s.operator LPCTSTR());
  s = ts1.Format(_T("%#M"));
  _tprintf(_T("Minutes in the timespan without leading zeros : %s\n"), s.operator LPCTSTR());
  s = ts1.Format(_T("%#S"));
  _tprintf(_T("Seconds in the timespan without leading zeros : %s\n"), s.operator LPCTSTR());

  //Try out of the CLTimeOfDay class

  //first the constructors
  CLTimeOfDay<> tod;
  ASSERT(!tod.IsValid());

  //tod = CLTimeOfDay(23, 59, 61, 0); //should assert
  //tod = CLTimeOfDay(24, 59, 61, 0); //should assert
  //tod = CLTimeOfDay(23, 59, 59, 1000); //should assert
                     
  SYSTEMTIME stnow;                   
  GetLocalTime(&stnow);
  tod = CLTimeOfDay<>(stnow);
  s = tod.Format();

  CLTimeOfDay<> tod2(tod);
  s = tod.Format();

  //tod2 = CLTimeOfDay(-23.5); //should assert
  //s = tod2.Format();

  //tod2 = CLTimeOfDay(-86401.5); //should assert
  //s = tod2.Format();

  //tod2 = CLTimeOfDay(86399.999);
  //s = tod2.Format();

  //tod2 = CLTimeOfDay(86199.9766);
  //s = tod2.Format();
  
  //Set functions tested via the constructors

  //The static constructors

  tod = CLTimeOfDay<>::CurrentLocalTime();
  s = tod.Format();
  _tprintf(_T("The local time of day is %s\n"), s.operator LPCTSTR());

  tod = CLTimeOfDay<>::CurrentSystemTime();
  s = tod.Format();
  _tprintf(_T("The UTC time of day is %s\n"), s.operator LPCTSTR());

  tod = CLTimeOfDay<>::Midnight();
  s = tod.Format();
  _tprintf(_T("Midnight as a time of day is %s\n"), s.operator LPCTSTR());

  tod = CLTimeOfDay<>::Midday();
  s = tod.Format();
  _tprintf(_T("Midday as a time of day is %s\n"), s.operator LPCTSTR());

  //try out the operations
  tod = CLTimeOfDay<>::CurrentSystemTime();
  WORD wVal = tod.GetHours();
  wVal = tod.GetMinutes();
  long lVal = tod.GetTotalSeconds();
  lVal;
  wVal = tod.GetSeconds();
  ASSERT(tod.IsValid());
  DWORD col = tod.Collate();
  _tprintf(_T("The collate function for the Current UTC time of day is\n"));
  _tprintf(_T("%ld\n"), col);
  bSuccess = tod.GetAsSystemTime(stnow);

  //Try out the GetAsTM accessor
  tod = CLTimeOfDay<>::CurrentSystemTime();
  bSuccess = tod.GetAsTM(tms);

  //Arithmetic operators
  
  CLTimeOfDay<> tod3(12, 1, 2);
  CLTimeSpan  ts4(1, 25, 59, 45);
  //operator= already tested
  CLTimeOfDay<> tod4 = tod3 + ts4;
  s = tod4.Format();
  CString s2 = tod3.Format();
  CString s3 = ts4.Format();
  _tprintf(_T("%s"), s2);
  _tprintf(_T(" + %s = "), s3.operator LPCTSTR());
  _tprintf(_T("%s\n"), s.operator LPCTSTR());

  tod2 = tod3 - ts4;
  s2 = tod3.Format();
  _tprintf(_T("%s"), s2);
  s3 = ts4.Format();
  _tprintf(_T(" - %s = "), s3.operator LPCTSTR());
  s = tod2.Format();
  _tprintf(_T("%s\n"), s.operator LPCTSTR());
 
  s2 = tod3.Format();
  _tprintf(_T("%s"), s2);
  tod3 += ts4;
  s3 = ts4.Format();
  _tprintf(_T(" += %s = "), s3.operator LPCTSTR());
  s = tod3.Format();
  _tprintf(_T("%s\n"), s.operator LPCTSTR());
  
  s2 = tod3.Format();
  _tprintf(_T("%s"), s2);
  tod3 -= ts4;
  s3 = ts4.Format();
  _tprintf(_T(" -= %s = "), s3.operator LPCTSTR());
  s = tod3.Format();
  _tprintf(_T("%s\n"), s.operator LPCTSTR());

  //try out negative timespans
  ts4.Negate();
  tod4 = tod3 + ts4;
  s2 = tod3.Format();
  _tprintf(_T("%s"), s2);
  s3 = ts4.Format();
  _tprintf(_T(" + %s = "), s3.operator LPCTSTR());
  s = tod4.Format();
  _tprintf(_T("%s\n"), s.operator LPCTSTR());

  tod2 = tod3 - ts4;
  s2 = tod3.Format();
  _tprintf(_T("%s"), s2);
  s3 = ts4.Format();
  _tprintf(_T(" - %s = "), s3.operator LPCTSTR());
  s = tod2.Format();
  _tprintf(_T("%s\n"), s.operator LPCTSTR());

  s2 = tod3.Format();
  _tprintf(_T("%s"), s2);
  tod3 += ts4;
  s3 = ts4.Format();
  _tprintf(_T(" += %s = "), s3.operator LPCTSTR());
  s = tod3.Format();
  _tprintf(_T("%s\n"), s.operator LPCTSTR());
  
  s2 = tod3.Format();
  _tprintf(_T("%s"), s2);
  tod3 -= ts4;
  s3 = ts4.Format();
  _tprintf(_T(" -= %s = "), s3.operator LPCTSTR());
  s = tod3.Format();
  _tprintf(_T("%s\n"), s.operator LPCTSTR());

  //equality operators

  CLTimeOfDay<> tod1(12, 1, 2);
  tod2 = CLTimeOfDay<>(13, 0, 2);

  ASSERT(tod1 <= tod2);
  ASSERT(tod1 < tod2);
  ASSERT(tod2 > tod1);
  ASSERT(tod2 >= tod1);
  ASSERT(tod2 != tod1);
  ASSERT(!(tod2 == tod1));

  //Full test of the Format function
  tod1 = CLTimeOfDay<>::CurrentLocalTime();
  s = tod1.Format();
  _tprintf(_T("A full test of CLTimeOfDay<>::Format for Now (LOCAL) gives\n"));
  s = tod1.Format(_T("%H"));
  _tprintf(_T("Hours in the current day : %s\n"), s.operator LPCTSTR());
  s = tod1.Format(_T("%h"));
  _tprintf(_T("AM / PM Hours in the current day : %s\n"), s.operator LPCTSTR());
  s = tod1.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the current day : %s\n"), s.operator LPCTSTR());
  s = tod1.Format(_T("%M"));
  _tprintf(_T("Minutes in the current day : %s\n"), s.operator LPCTSTR());
  s = tod1.Format(_T("%S"));
  _tprintf(_T("Seconds in the current day : %s\n"), s.operator LPCTSTR());
  s = tod1.Format(_T("%#H"));
  _tprintf(_T("Hours in the current day without leading zeros: %s\n"), s.operator LPCTSTR());
  s = tod1.Format(_T("%#h"));
  _tprintf(_T("AM / PM Hours in the current day without leading zeros: %s\n"), s.operator LPCTSTR());
  s = tod1.Format(_T("%#M"));
  _tprintf(_T("Minutes in the current day without leading zeros : %s\n"), s.operator LPCTSTR());
  s = tod1.Format(_T("%#S"));
  _tprintf(_T("Seconds in the current day without leading zeros : %s\n"), s.operator LPCTSTR());

  //Try out the CLDate class

  //first the constructors
  CLDate<CGregorianCalendar> xlneg(3, January, 2, 0, 0, 0);
  CDate<CGregorianCalendar> ld = xlneg.GetCDate();
  s = ld.Format();
  CLTimeOfDay<> ltod = xlneg.GetCLTimeOfDay();
  CString sneg = xlneg.Format();
  xlneg = CLDate<CGregorianCalendar>(3, January, 2, 0, 0, 0);
  sneg = xlneg.Format();
  xlneg = CLDate<CGregorianCalendar>(3, January, 2, 4, 3, 2);
  sneg = xlneg.Format();
  xlneg = CLDate<CGregorianCalendar>(3, January, 2, 23, 59, 59);
  sneg = xlneg.Format();

  CLDate<CGregorianCalendar> ldate;
  ASSERT(!ldate.IsValid());
  ldate = CLDate<CGregorianCalendar>(1995, November, 3, 12, 11, 10);
  s = ldate.Format();
  ::GetLocalTime(&st);
  ldate = CLDate<CGregorianCalendar>(st);
  s = ldate.Format();
  ldate = CLDate<CGregorianCalendar>(1995, March, 5, Sunday, 0, 0, 0); 
  s = ldate.Format();
  CLDate<CJulianCalendar> ldate2 = CLDate<CJulianCalendar>(1000, EPOCH_JD, 0, 0, 0);
  s = ldate2.Format();
  //Copy constructor already tested

  CTime ct = CTime::GetCurrentTime();
  ldate = CLDate<CGregorianCalendar>(ct);
  s = ldate.Format();
  ldate = CLDate<CGregorianCalendar>(CDate<CGregorianCalendar>::CurrentLocalDate(), CLTimeOfDay<>::CurrentLocalTime());
  s = ldate.Format();

  //Construction using the Win32 FILETIME structure
  GetLocalTime(&st);
  SystemTimeToFileTime(&st, &ft);
  ldate = CLDate<CGregorianCalendar>(ft);
  s = ldate.Format();
      
  COleDateTime ldt(1992, October, 10, 3, 4, 5);
  ldate = CLDate<CGregorianCalendar>(ldt);
  s = ldate.Format();
  ldt = COleDateTime(1970, February, 1, 5, 6, 7);
  ldate = CLDate<CGregorianCalendar>(ldt);
  s = ldate.Format();
  
  //Set operators tested via the constructors above

  //Static Constructors
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  s = ldate.Format();   

  //Try out the GetAsTM accessor
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  bSuccess = ldate.GetAsTM(tms);

  //Need to try out the following functions
  //GetCDate & GetCLTimeOfDay tested through the Format function
  bSuccess = ldate.GetAsSystemTime(st);
  bSuccess = ldate.GetAsFileTime(ft);
  bSuccess = ldate.GetAsCOleDateTime(dt);
  
  ldate.AddYear(10);
  s = ldate.Format();
  _tprintf(_T("10 years from now UTC is %s\n"), s.operator LPCTSTR());

  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  ldate.AddWeek(-10);
  s = ldate.Format();
  _tprintf(_T("10 weeks before today UTC is %s\n"), s.operator LPCTSTR());

  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  ldate.AddMonth(-14);
  s = ldate.Format();
  _tprintf(_T("14 months before today UTC is %s\n"), s.operator LPCTSTR());
  
  ASSERT(ldate.IsValid());     
  ldate = CLDate<CGregorianCalendar>();
  ASSERT(!ldate.IsValid());

  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  s = ldate.Format();
  _tprintf(_T("The current UTC date is %s\n"), s.operator LPCTSTR());
  ldate = CLDate<CGregorianCalendar>::CurrentLocalTime();
  s = ldate.Format();
  _tprintf(_T("The current LOCAL date is %s\n"), s.operator LPCTSTR());

  //operator= already tried out

  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  ts = CTimeSpan(10, 10, 10, 10);
  ldate += ts;
  s = ts.Format();
  _tprintf(_T("%s from today UTC is "), s);
  s = ldate.Format();
  _tprintf(_T("%s\n"), s.operator LPCTSTR());

  //Arithmetic operators
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  CLDate<CGregorianCalendar> ldate3 = ldate;
  s = ldate3.Format();
  ldate3 += CLTimeSpan(0, 1, 0, 0);
  s = ldate3.Format();
  ldate3 -= CLTimeSpan(0, 1, 0, 0);
  s = ldate3.Format();
  ts = ldate3 - ldate;
  s = ldate3.Format();
  ++ldate3;
  s = ldate3.Format();
  --ldate3;
  s = ldate3.Format();

  //try out the equality operators
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  CLDate<CGregorianCalendar> ldate4(ldate);
  ASSERT(ldate == ldate4);
  ASSERT(!(ldate != ldate4));
  ldate4 += CLTimeSpan(0, 0, 0, 1);
  ASSERT(ldate4 != ldate);
  ASSERT(ldate4 >= ldate);
  ASSERT(ldate <= ldate4);
  ASSERT(ldate < ldate4);

  //AssertValid() already tested internally

  ldate = CLDate<CGregorianCalendar>::CurrentLocalTime();
  sString = ldate.Format();
  ldate3 = ldate.AsGregorianCLDate();
  s = ldate3.Format();
  ldate2 = ldate.AsJulianCLDate();
  s = ldate2.Format();
  
  //Full test of the Format function
  ldate = CLDate<CGregorianCalendar>::CurrentLocalTime();
  s = ldate.Format();
  _tprintf(_T("A full test of CLDate::Format for Now (LOCAL) gives\n"));
  s = ldate.Format(_T("%a"));
  _tprintf(_T("Abbreviated weekday name : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%A"));
  _tprintf(_T("Full weekday name : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%b"));
  _tprintf(_T("Abbreviated month name : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%B"));
  _tprintf(_T("Full month name : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%d"));
  _tprintf(_T("Day of month as decimal number (01 - 31) : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%j"));
  _tprintf(_T("Day of year as decimal number (001 - 366) : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%m"));
  _tprintf(_T("Month as decimal number (01 - 12) : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%U"));
  _tprintf(_T("Week of year as decimal number : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%w"));
  _tprintf(_T("Weekday as decimal number (1 - 7), Sunday is 1 : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%x"));
  _tprintf(_T("Date representation for current locale : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%y"));
  _tprintf(_T("Year without century, as decimal number (00 - 99) : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%Y"));
  _tprintf(_T("Year with century, as decimal number : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#x"));
  _tprintf(_T("Long date representation, appropriate to current locale : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#d"));
  _tprintf(_T("Day of month without leading zeros : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#j"));
  _tprintf(_T("Day of year without leading zeros : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#m"));
  _tprintf(_T("Month without leading zeroes : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#U"));
  _tprintf(_T("Week of year without leading zeroes, with Sunday as first day of week : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#y"));
  _tprintf(_T("Year without century without leading zeroes : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%H"));
  _tprintf(_T("Hours in the current day : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%h"));
  _tprintf(_T("AM / PM Hours in the current day : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%P"));
  _tprintf(_T("AM / PM Indicator for the current day : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%M"));
  _tprintf(_T("Minutes in the current day : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%S"));
  _tprintf(_T("Seconds in the current day : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#H"));
  _tprintf(_T("Hours in the current day without leading zeros: %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#h"));
  _tprintf(_T("AM / PM Hours in the current day without leading zeros: %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#M"));
  _tprintf(_T("Minutes in the current day without leading zeros : %s\n"), s.operator LPCTSTR());
  s = ldate.Format(_T("%#S"));
  _tprintf(_T("Seconds in the current day without leading zeros : %s\n"), s.operator LPCTSTR());
  ldate = CLDate<CGregorianCalendar>::CurrentSystemTime();
  s = ldate.Format();
  _tprintf(_T("The Current UTC long date is %s\n"), s.operator LPCTSTR());

  //Test of the Collate and SetFromCollate functions
  {
    CDate<CGregorianCalendar> k = CDate<CGregorianCalendar>::CurrentLocalDate();
    LONG c = k.Collate();
    k.SetFromCollate(c);
    CLTimeOfDay<> f = CLTimeOfDay<>(1, 2, 3);
    DWORD j = f.Collate();
    f.SetFromCollate(j);
    CLDate<CGregorianCalendar> g = CLDate<CGregorianCalendar>::CurrentLocalTime();
  }

  //Display some intro text
  _tprintf(_T("\nThis program is now going to test out the serialization support\n"));
  _tprintf(_T("by creating a file C:\\DTIME+Archive.date and reading and writing to it.\n"));
  _tprintf(_T("The file size should end up as 34 bytes\n"));
  _tprintf(_T("<Hit Enter to continue>\n"));
  chr = getchar();

  //Test out the persistence methods
	CFile datafile;
  if (datafile.Open(_T("C:\\DTIME+Archive.dat"), CFile::modeWrite | CFile::modeCreate))
  {
    CArchive archive(&datafile, CArchive::store);

    archive << x;
    _tprintf(_T("Saving %s to archive\n"), x.Format().operator LPCTSTR());
    archive << ts;
    _tprintf(_T("Saving %s to archive\n"), ts.Format().operator LPCTSTR());
    archive << tod;
    _tprintf(_T("Saving %s to archive\n"), tod.Format().operator LPCTSTR());
    archive << ldate;
    _tprintf(_T("Saving %s to archive\n"), ldate.Format().operator LPCTSTR());
  }
  datafile.Close();
  if (datafile.Open(_T("C:\\DTIME+Archive.dat"), CFile::modeRead))
  {
    CArchive archive(&datafile, CArchive::load);

    CDate<CGregorianCalendar> xin;
    archive >> xin;
    _tprintf(_T("Loaded %s from archive\n"), xin.Format().operator LPCTSTR());
    CLTimeSpan tsin;
    archive >> tsin;
    _tprintf(_T("Loaded %s from archive\n"), tsin.Format().operator LPCTSTR());
    CLTimeOfDay<> todin;
    archive >> todin;
    _tprintf(_T("Loaded %s from archive\n"), todin.Format().operator LPCTSTR());
    CLDate<CGregorianCalendar> ldatein;
    archive >> ldatein;
    _tprintf(_T("Loaded %s from archive\n"), ldatein.Format().operator LPCTSTR());
  }

  DWORD TimeInterval = GetTickCount() - StartTicks;
  _tprintf(_T("\nTotal Time taken to execute program in milliseconds was : %ld\n"), TimeInterval);

	return 0;
}


